@@ -1,9 +1,13 @@
 extern crate krill;
 
-use krill::cli::options::Options;
-use krill::cli::report::ReportFormat;
-use krill::cli::{Error, KrillClient};
-use krill::commons::util::httpclient;
+use krill::{
+    cli::{
+        options::Options,
+        report::ReportFormat,
+        {Error, KrillClient},
+    },
+    commons::util::httpclient,
+};
 
 #[tokio::main]
 async fn main() {
@@ -15,7 +19,7 @@ async fn main() {
                 Err(e) => {
                     if format != ReportFormat::None {
                         match &e {
-                            Error::HttpClientError(httpclient::Error::ErrorWithJson(_code, res)) => {
+                            Error::HttpClientError(httpclient::Error::ErrorResponseWithJson(_uri, _code, res)) => {
                                 if format == ReportFormat::Json {
                                     eprintln!("{}", e);
                                 } else if let Some(delta_error) = res.delta_error() {
