@@ -3,16 +3,19 @@ use std::str::FromStr;
 
 use serde::Serialize;
 
-use crate::commons::api::{
-    AllCertAuthIssues, CaCommandDetails, CaRepoDetails, CertAuthInfo, CertAuthIssues, CertAuthList, ChildCaInfo,
-    CommandHistory, ParentCaContact, ParentStatuses, PublisherDetails, PublisherList, RepoStatus, RoaDefinitions,
-    RtaList, RtaPrepResponse, ServerInfo,
+use crate::{
+    commons::{
+        api::{
+            AllCertAuthIssues, AspaDefinitionList, CaCommandDetails, CaRepoDetails, CertAuthInfo, CertAuthIssues,
+            CertAuthList, ChildCaInfo, ChildrenConnectionStats, CommandHistory, ParentCaContact, ParentStatuses,
+            PublisherDetails, PublisherList, RepoStatus, RoaDefinitions, RtaList, RtaPrepResponse, ServerInfo,
+        },
+        bgp::{BgpAnalysisAdvice, BgpAnalysisReport, BgpAnalysisSuggestion},
+        remote::{api::ClientInfos, rfc8183},
+    },
+    daemon::ca::ResourceTaggedAttestation,
+    pubd::RepoStats,
 };
-use crate::commons::bgp::{BgpAnalysisAdvice, BgpAnalysisReport, BgpAnalysisSuggestion};
-use crate::commons::remote::api::ClientInfos;
-use crate::commons::remote::rfc8183;
-use crate::daemon::ca::ResourceTaggedAttestation;
-use crate::pubd::RepoStats;
 
 //------------ ApiResponse ---------------------------------------------------
 
@@ -27,15 +30,21 @@ pub enum ApiResponse {
     CertAuthHistory(CommandHistory),
     CertAuthAction(CaCommandDetails),
     CertAuths(CertAuthList),
+
+    // ROA related
     RouteAuthorizations(RoaDefinitions),
     BgpAnalysisAdvice(BgpAnalysisAdvice),
     BgpAnalysisFull(BgpAnalysisReport),
     BgpAnalysisSuggestions(BgpAnalysisSuggestion),
 
+    // ASPA related
+    AspaDefinitions(AspaDefinitionList),
+
     ParentCaContact(ParentCaContact),
     ParentStatuses(ParentStatuses),
 
     ChildInfo(ChildCaInfo),
+    ChildrenStats(ChildrenConnectionStats),
 
     PublisherDetails(PublisherDetails),
     PublisherList(PublisherList),
@@ -78,9 +87,11 @@ impl ApiResponse {
                 ApiResponse::BgpAnalysisAdvice(analysis) => Ok(Some(analysis.report(fmt)?)),
                 ApiResponse::BgpAnalysisFull(table) => Ok(Some(table.report(fmt)?)),
                 ApiResponse::BgpAnalysisSuggestions(suggestions) => Ok(Some(suggestions.report(fmt)?)),
+                ApiResponse::AspaDefinitions(definitions) => Ok(Some(definitions.report(fmt)?)),
                 ApiResponse::ParentCaContact(contact) => Ok(Some(contact.report(fmt)?)),
                 ApiResponse::ParentStatuses(statuses) => Ok(Some(statuses.report(fmt)?)),
                 ApiResponse::ChildInfo(info) => Ok(Some(info.report(fmt)?)),
+                ApiResponse::ChildrenStats(stats) => Ok(Some(stats.report(fmt)?)),
                 ApiResponse::PublisherList(list) => Ok(Some(list.report(fmt)?)),
                 ApiResponse::PublisherDetails(details) => Ok(Some(details.report(fmt)?)),
                 ApiResponse::RepoStats(stats) => Ok(Some(stats.report(fmt)?)),
@@ -177,6 +188,7 @@ impl Report for CaCommandDetails {}
 impl Report for PublisherList {}
 
 impl Report for RepoStats {}
+impl Report for ChildrenConnectionStats {}
 
 impl Report for PublisherDetails {}
 
@@ -192,6 +204,8 @@ impl Report for BgpAnalysisAdvice {}
 impl Report for BgpAnalysisReport {}
 impl Report for BgpAnalysisSuggestion {}
 
+impl Report for AspaDefinitionList {}
+
 impl Report for CaRepoDetails {}
 impl Report for RepoStatus {}
 
