@@ -773,7 +773,15 @@ pub struct Revocations(Vec<Revocation>);
 
 impl Revocations {
     pub fn to_crl_entries(&self) -> Vec<CrlEntry> {
-        self.0.iter().map(|r| CrlEntry::new(r.serial, r.expires)).collect()
+        // Todo: include the revocation time in ['Revocation'] and use that.
+        //       this will require that we reprocess history to get this
+        //       value. We do know when an object was removed or replaced,
+        //       so we can get it - but it's not entirely trivial.
+        //
+        // See issue #788. This issue was added to the 0.10.0 backlog.
+        // For now we just do a quick hack and use 'now' rather than the
+        // future dated 'expires' time for the CRL entry.
+        self.0.iter().map(|r| CrlEntry::new(r.serial, Time::now())).collect()
     }
 
     /// Purges all expired revocations, and returns them.
@@ -1169,6 +1177,10 @@ impl fmt::Display for ResourceSetDiff {
 
 #[derive(Clone, Debug, Deserialize, Eq, PartialEq, Serialize)]
 pub struct CertAuthList {
+    // Even though we only have 1 field, we chose not to use a tuple struct here
+    // to allow for future extensions more easily.. we could then just add new
+    // fields and associated JSON members without affecting consumers of the API
+    // too much.
     cas: Vec<CertAuthSummary>,
 }
 
@@ -1192,6 +1204,12 @@ impl fmt::Display for CertAuthList {
     }
 }
 
+impl AsRef<Vec<CertAuthSummary>> for CertAuthList {
+    fn as_ref(&self) -> &Vec<CertAuthSummary> {
+        &self.cas
+    }
+}
+
 //------------ CertAuthSummary -----------------------------------------------
 
 #[derive(Clone, Debug, Deserialize, Eq, PartialEq, Serialize)]
@@ -1327,27 +1345,7 @@ impl ParentStatuses {
         sorted_parents.into_iter().map(|(handle, _)| handle).cloned().collect()
     }
 
-    pub fn set_failure(&mut self, parent: &ParentHandle, uri: &ServiceUri, error: ErrorResponse, next_seconds: i64) {
-        self.get_mut_status(parent)
-            .set_failure(uri.clone(), error, next_seconds);
-    }
-
-    pub fn set_entitlements(
-        &mut self,
-        parent: &ParentHandle,
-        uri: &ServiceUri,
-        entitlements: &Entitlements,
-        next_seconds: i64,
-    ) {
-        self.get_mut_status(parent)
-            .set_entitlements(uri.clone(), entitlements, next_seconds);
-    }
-
-    pub fn set_last_updated(&mut self, parent: &ParentHandle, uri: &ServiceUri, next_seconds: i64) {
-        self.get_mut_status(parent).set_last_updated(uri.clone(), next_seconds);
-    }
-
-    fn get_mut_status(&mut self, parent: &ParentHandle) -> &mut ParentStatus {
+    pub fn get_mut_status(&mut self, parent: &ParentHandle) -> &mut ParentStatus {
         if !self.0.contains_key(parent) {
             self.0.insert(parent.clone(), ParentStatus::default());
         }
@@ -1358,6 +1356,10 @@ impl ParentStatuses {
     pub fn remove(&mut self, parent: &ParentHandle) {
         self.0.remove(parent);
     }
+
+    pub fn insert(&mut self, parent: ParentHandle, status: ParentStatus) {
+        self.0.insert(parent, status);
+    }
 }
 
 impl IntoIterator for ParentStatuses {
@@ -1385,11 +1387,6 @@ impl fmt::Display for ParentStatuses {
                     writeln!(f, "URI: {}", exchange.uri)?;
                     writeln!(f, "Status: {}", exchange.result)?;
                     writeln!(f, "Last contacted: {}", exchange.timestamp().to_rfc3339())?;
-                    writeln!(
-                        f,
-                        "Next contact on or before: {}",
-                        status.next_exchange_before().to_rfc3339()
-                    )?;
 
                     if exchange.was_success() {
                         write!(f, "Resource Entitlements:")?;
@@ -1485,16 +1482,11 @@ impl From<&IssuedCert> for ParentStatusCert {
 pub struct ParentStatus {
     last_exchange: Option<ParentExchange>,
     last_success: Option<Timestamp>,
-    next_exchange_before: Timestamp,
     all_resources: ResourceSet,
     entitlements: HashMap<ResourceClassName, KnownEntitlement>,
 }
 
 impl ParentStatus {
-    fn next_exchange_before(&self) -> Timestamp {
-        self.next_exchange_before
-    }
-
     pub fn last_success(&self) -> Option<Timestamp> {
         self.last_success
     }
@@ -1511,21 +1503,16 @@ impl ParentStatus {
         self.last_exchange.as_ref().map(|e| e.to_failure_opt()).flatten()
     }
 
-    fn set_next_exchange(&mut self, next_run_seconds: i64) {
-        self.next_exchange_before = Timestamp::now_plus_seconds(next_run_seconds);
-    }
-
-    fn set_failure(&mut self, uri: ServiceUri, error: ErrorResponse, next_run_seconds: i64) {
+    pub fn set_failure(&mut self, uri: ServiceUri, error: ErrorResponse) {
         self.last_exchange = Some(ParentExchange {
             timestamp: Timestamp::now(),
             uri,
             result: ExchangeResult::Failure(error),
         });
-        self.set_next_exchange(next_run_seconds);
     }
 
-    fn set_entitlements(&mut self, uri: ServiceUri, entitlements: &Entitlements, next_run_seconds: i64) {
-        self.set_last_updated(uri, next_run_seconds);
+    pub fn set_entitlements(&mut self, uri: ServiceUri, entitlements: &Entitlements) {
+        self.set_last_updated(uri);
 
         self.entitlements = entitlements
             .classes()
@@ -1543,10 +1530,9 @@ impl ParentStatus {
         }
 
         self.all_resources = all_resources;
-        self.set_next_exchange(next_run_seconds);
     }
 
-    fn set_last_updated(&mut self, uri: ServiceUri, next_run_seconds: i64) {
+    pub fn set_last_updated(&mut self, uri: ServiceUri) {
         let timestamp = Timestamp::now();
         self.last_exchange = Some(ParentExchange {
             timestamp,
@@ -1554,7 +1540,6 @@ impl ParentStatus {
             result: ExchangeResult::Success,
         });
         self.last_success = Some(timestamp);
-        self.set_next_exchange(next_run_seconds);
     }
 }
 
@@ -1564,7 +1549,6 @@ impl Default for ParentStatus {
             last_exchange: None,
             last_success: None,
             all_resources: ResourceSet::default(),
-            next_exchange_before: Timestamp::now() + Duration::hours(1),
             entitlements: HashMap::new(),
         }
     }
@@ -1910,7 +1894,13 @@ impl ChildExchange {
 
     pub fn is_krill_above_0_9_1(&self) -> bool {
         if let Some(agent) = &self.user_agent {
-            if let Some(version) = agent.strip_prefix("krill/") {
+            // local-child is used by local children, it is extremely
+            // unlikely that they would become suspend candidates in
+            // the real world - but.. we have to use these to test the
+            // auto-suspend logic in the high-level "suspend.rs" test
+            if agent == "local-child" {
+                return true;
+            } else if let Some(version) = agent.strip_prefix("krill/") {
                 if let Ok(krill_version) = KrillVersion::from_str(version) {
                     return krill_version > KrillVersion::release(0, 9, 1);
                 }
@@ -1927,6 +1917,10 @@ impl ChildExchange {
 pub struct Timestamp(i64);
 
 impl Timestamp {
+    pub fn new(ts: i64) -> Self {
+        Timestamp(ts)
+    }
+
     pub fn now() -> Self {
         Timestamp(Time::now().timestamp())
     }
@@ -2889,7 +2883,9 @@ mod test {
         use crate::commons::error::Error;
         use crate::commons::util::httpclient;
 
-        issues.add_repo_issue(Error::HttpClientError(httpclient::Error::Forbidden).to_error_response());
+        issues.add_repo_issue(
+            Error::HttpClientError(httpclient::Error::forbidden("https://example.com/")).to_error_response(),
+        );
         issues.add_parent_issue(
             Handle::from_str("parent").unwrap(),
             Error::Rfc6492SignatureInvalid.to_error_response(),
@@ -2982,7 +2978,6 @@ mod test {
     fn find_sync_candidates() {
         let uri = ServiceUri::try_from("https://example.com/rfc6492/child/".to_string()).unwrap();
 
-        let in_five_seconds = Timestamp::now_plus_seconds(5);
         let five_seconds_ago = Timestamp::now_minus_seconds(5);
         let five_mins_ago = Timestamp::now_minus_seconds(300);
 
@@ -2996,7 +2991,6 @@ mod test {
         let p3_status_no_exchange = ParentStatus {
             last_exchange: None,
             last_success: None,
-            next_exchange_before: in_five_seconds,
             all_resources: ResourceSet::default(),
             entitlements: HashMap::new(),
         };
@@ -3008,7 +3002,6 @@ mod test {
                 result: ExchangeResult::Success,
             }),
             last_success: None,
-            next_exchange_before: in_five_seconds,
             all_resources: ResourceSet::default(),
             entitlements: HashMap::new(),
         };
@@ -3020,7 +3013,6 @@ mod test {
                 result: ExchangeResult::Failure(ErrorResponse::new("err", "err!")),
             }),
             last_success: None,
-            next_exchange_before: in_five_seconds,
             all_resources: ResourceSet::default(),
             entitlements: HashMap::new(),
         };
@@ -3032,7 +3024,6 @@ mod test {
                 result: ExchangeResult::Success,
             }),
             last_success: None,
-            next_exchange_before: in_five_seconds,
             all_resources: ResourceSet::default(),
             entitlements: HashMap::new(),
         };
