@@ -1,23 +1,24 @@
-use std::collections::BTreeMap;
-use std::fmt;
-use std::str::FromStr;
+use std::{collections::BTreeMap, fmt, str::FromStr};
 
 use chrono::{DateTime, NaiveDateTime, SecondsFormat, Utc};
 
-use rpki::{crypto::KeyIdentifier, x509::Time};
+use rpki::repository::{crypto::KeyIdentifier, x509::Time};
 
 use crate::{
     commons::{
         api::{
-            ArgKey, ArgVal, ChildHandle, Handle, Label, Message, ParentHandle, PublisherHandle, RequestResourceLimit,
-            ResourceClassName, ResourceSet, RevocationRequest, RoaDefinitionUpdates, RtaName, StorableParentContact,
+            ArgKey, ArgVal, AspaCustomer, AspaProvidersUpdate, ChildHandle, Handle, Label, Message, ParentHandle,
+            PublisherHandle, RequestResourceLimit, ResourceClassName, ResourceSet, RevocationRequest,
+            RoaDefinitionUpdates, RtaName, StorableParentContact,
         },
         eventsourcing::{CommandKey, CommandKeyError, StoredCommand, WithStorableDetails},
         remote::rfc8183::ServiceUri,
     },
-    daemon::ca,
+    daemon::ca::{self, DropReason},
 };
 
+use super::AspaDefinitionUpdates;
+
 //------------ CaCommandDetails ----------------------------------------------
 #[derive(Clone, Debug, Deserialize, Eq, PartialEq, Serialize)]
 pub struct CaCommandDetails {
@@ -428,6 +429,12 @@ pub enum StorableCaCommand {
     ChildRemove {
         child: ChildHandle,
     },
+    ChildSuspendInactive {
+        child: ChildHandle,
+    },
+    ChildUnsuspend {
+        child: ChildHandle,
+    },
     GenerateNewIdKey,
     AddParent {
         parent: ParentHandle,
@@ -448,6 +455,10 @@ pub enum StorableCaCommand {
         resource_class_name: ResourceClassName,
         resources: ResourceSet,
     },
+    DropResourceClass {
+        resource_class_name: ResourceClassName,
+        reason: DropReason,
+    },
     KeyRollInitiate {
         older_than_seconds: i64,
     },
@@ -461,6 +472,17 @@ pub enum StorableCaCommand {
         updates: RoaDefinitionUpdates,
     },
     ReissueBeforeExpiring,
+    ForceReissue,
+    AspasUpdate {
+        updates: AspaDefinitionUpdates,
+    },
+    AspasUpdateExisting {
+        customer: AspaCustomer,
+        update: AspaProvidersUpdate,
+    },
+    AspaRemove {
+        customer: AspaCustomer,
+    },
     RepoUpdate {
         service_uri: ServiceUri,
     },
@@ -510,6 +532,12 @@ impl WithStorableDetails for StorableCaCommand {
             StorableCaCommand::ChildRemove { child } => {
                 CommandSummary::new("cmd-ca-child-remove", &self).with_child(child)
             }
+            StorableCaCommand::ChildSuspendInactive { child } => {
+                CommandSummary::new("cmd-ca-child-suspend-inactive", &self).with_child(child)
+            }
+            StorableCaCommand::ChildUnsuspend { child } => {
+                CommandSummary::new("cmd-ca-child-unsuspend", &self).with_child(child)
+            }
             StorableCaCommand::ChildRevokeKey { child, revoke_req } => {
                 CommandSummary::new("cmd-ca-child-revoke", &self)
                     .with_child(child)
@@ -537,6 +565,14 @@ impl WithStorableDetails for StorableCaCommand {
             } => CommandSummary::new("cmd-ca-rcn-receive", &self)
                 .with_rcn(resource_class_name)
                 .with_resources(resources),
+            StorableCaCommand::DropResourceClass {
+                resource_class_name,
+                reason,
+            } => CommandSummary::new("cmd-ca-rc-drop", &self)
+                .with_rcn(resource_class_name)
+                .with_arg("reason", reason),
+
+            // Key rolls
             StorableCaCommand::KeyRollInitiate { older_than_seconds } => {
                 CommandSummary::new("cmd-ca-keyroll-init", &self).with_seconds(*older_than_seconds)
             }
@@ -546,14 +582,24 @@ impl WithStorableDetails for StorableCaCommand {
             StorableCaCommand::KeyRollFinish { resource_class_name } => {
                 CommandSummary::new("cmd-ca-keyroll-finish", &self).with_rcn(resource_class_name)
             }
+
+            // ROA
             StorableCaCommand::RoaDefinitionUpdates { updates } => CommandSummary::new("cmd-ca-roas-updated", &self)
                 .with_added(updates.added().len())
                 .with_removed(updates.removed().len()),
+
+            // ASPA
+            StorableCaCommand::AspasUpdate { .. } => CommandSummary::new("cmd-ca-aspas-update", &self),
+            StorableCaCommand::AspasUpdateExisting { .. } => CommandSummary::new("cmd-ca-aspas-update-existing", &self),
+            StorableCaCommand::AspaRemove { .. } => CommandSummary::new("cmd-ca-aspas-remove", &self),
+
+            // REPO
             StorableCaCommand::RepoUpdate { service_uri } => {
                 CommandSummary::new("cmd-ca-repo-update", &self).with_service_uri(service_uri)
             }
 
             StorableCaCommand::ReissueBeforeExpiring => CommandSummary::new("cmd-ca-reissue-before-expiring", &self),
+            StorableCaCommand::ForceReissue => CommandSummary::new("cmd-ca-force-reissue", &self),
 
             // RTA
             StorableCaCommand::RtaPrepare { name } => {
@@ -605,7 +651,13 @@ impl fmt::Display for StorableCaCommand {
                 revoke_req.class_name()
             ),
             StorableCaCommand::ChildRemove { child } => {
-                write!(f, "Remove child '{}' and revoke&remove its certs", child)
+                write!(f, "Remove child '{}' and revoke & remove its certs", child)
+            }
+            StorableCaCommand::ChildSuspendInactive { child } => {
+                write!(f, "Suspend inactive child '{}': stop publishing its certs", child)
+            }
+            StorableCaCommand::ChildUnsuspend { child } => {
+                write!(f, "Unsuspend child '{}': publish its unexpired certs", child)
             }
 
             // ------------------------------------------------------------
@@ -640,6 +692,14 @@ impl fmt::Display for StorableCaCommand {
                 resource_class_name,
                 resources.summary()
             ),
+            StorableCaCommand::DropResourceClass {
+                resource_class_name,
+                reason,
+            } => write!(
+                f,
+                "Removing resource class '{}' because of reason: {}",
+                resource_class_name, reason
+            ),
 
             // ------------------------------------------------------------
             // Key rolls
@@ -685,6 +745,22 @@ impl fmt::Display for StorableCaCommand {
             StorableCaCommand::ReissueBeforeExpiring => {
                 write!(f, "Automatically re-issue objects before they would expire")
             }
+            StorableCaCommand::ForceReissue => {
+                write!(f, "Force re-issuance of objects")
+            }
+
+            // ------------------------------------------------------------
+            // ASPA Support
+            // ------------------------------------------------------------
+            StorableCaCommand::AspasUpdate { updates } => {
+                write!(f, "{}", updates)
+            }
+            StorableCaCommand::AspasUpdateExisting { customer, update } => {
+                write!(f, "update ASPA for customer AS: {} {}", customer, update)
+            }
+            StorableCaCommand::AspaRemove { customer } => {
+                write!(f, "Remove ASPA for customer AS: {}", customer)
+            }
 
             // ------------------------------------------------------------
             // Publishing
