@@ -3,6 +3,9 @@
 mod admin;
 pub use self::admin::*;
 
+mod aspa;
+pub use self::aspa::*;
+
 mod ca;
 pub use self::ca::*;
 
@@ -20,22 +23,19 @@ pub use self::roas::*;
 
 pub mod rrdp;
 
-use std::collections::HashMap;
-use std::fmt;
-use std::sync::Arc;
+use std::{collections::HashMap, fmt, sync::Arc};
 
 use bytes::Bytes;
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 
-use rpki::cert::Cert;
-use rpki::crl::Crl;
-use rpki::crypto::KeyIdentifier;
-use rpki::manifest::Manifest;
-use rpki::roa::Roa;
+use rpki::repository::{
+    aspa::Aspa, cert::Cert, crl::Crl, crypto::KeyIdentifier, manifest::Manifest, resources::AsId, roa::Roa,
+};
 
-use crate::commons::error::RoaDeltaError;
-use crate::commons::util::sha256;
-use crate::daemon::ca::RouteAuthorization;
+use crate::{
+    commons::{error::RoaDeltaError, util::sha256},
+    daemon::ca::RouteAuthorization,
+};
 
 // Some syntactic sugar to help this old coder's brain deal with the mess of Strings
 pub type Message = String;
@@ -100,6 +100,12 @@ impl From<&Roa> for Base64 {
     }
 }
 
+impl From<&Aspa> for Base64 {
+    fn from(aspa: &Aspa) -> Self {
+        Base64::from_content(&aspa.to_captured().into_bytes())
+    }
+}
+
 impl From<&Manifest> for Base64 {
     fn from(mft: &Manifest) -> Self {
         Base64::from_content(&mft.to_captured().into_bytes())
@@ -227,6 +233,8 @@ pub struct ErrorResponse {
     args: HashMap<String, String>,
     #[serde(skip_serializing_if = "Option::is_none")]
     delta_error: Option<RoaDeltaError>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    aspa_providers_update_conflict: Option<AspaProvidersUpdateConflict>,
 }
 
 impl ErrorResponse {
@@ -236,6 +244,7 @@ impl ErrorResponse {
             msg: msg.to_string(),
             args: HashMap::new(),
             delta_error: None,
+            aspa_providers_update_conflict: None,
         }
     }
 
@@ -286,11 +295,20 @@ impl ErrorResponse {
         res
     }
 
+    pub fn with_asn(self, asn: AsId) -> Self {
+        self.with_arg("asn", asn)
+    }
+
     pub fn with_roa_delta_error(mut self, roa_delta_error: &RoaDeltaError) -> Self {
         self.delta_error = Some(roa_delta_error.clone());
         self
     }
 
+    pub fn with_aspa_providers_conflict(mut self, conflict: &AspaProvidersUpdateConflict) -> Self {
+        self.aspa_providers_update_conflict = Some(conflict.clone());
+        self
+    }
+
     pub fn with_key_identifier(self, ki: &KeyIdentifier) -> Self {
         self.with_arg("key_id", ki)
     }
