@@ -1,23 +1,27 @@
 //! Data objects used in the (RRDP) repository. I.e. the publish, update, and
 //! withdraw elements, as well as the notification, snapshot and delta file
 //! definitions.
-use std::fmt;
-use std::path::PathBuf;
-use std::{collections::HashMap, path::Path};
+use std::{
+    fmt,
+    path::PathBuf,
+    {collections::HashMap, path::Path},
+};
 
 use bytes::Bytes;
 use chrono::Duration;
 use serde::{de, Deserialize, Deserializer, Serialize, Serializer};
 use uuid::Uuid;
 
-use rpki::uri;
-use rpki::x509::Time;
+use rpki::{repository::x509::Time, uri};
 
-use crate::commons::api::Base64;
-use crate::commons::api::HexEncodedHash;
-use crate::commons::util::file;
-use crate::commons::util::xml::XmlWriter;
-use crate::commons::{api::publication, error::KrillIoError};
+use crate::{
+    commons::{
+        api::{publication, Base64, HexEncodedHash},
+        error::KrillIoError,
+        util::{file, xml::XmlWriter},
+    },
+    constants::RRDP_FIRST_SERIAL,
+};
 
 const VERSION: &str = "1";
 const NS: &str = "http://www.ripe.net/rpki/rrdp";
@@ -322,36 +326,13 @@ impl NotificationUpdate {
 }
 
 impl Notification {
-    pub fn update_old_way(&mut self, update: NotificationUpdate) {
-        let (time, session_opt, snapshot, delta, last_delta) = update.unwrap();
-        if let Some(session) = session_opt {
-            self.session = session;
-        }
-
-        self.serial += 1;
-        self.time = time;
-
-        let mut refs_to_retire = vec![(Time::now(), self.snapshot.clone())];
-
-        self.snapshot = snapshot;
-
-        for d in &self.deltas {
-            if d.serial < last_delta {
-                refs_to_retire.push((Time::now(), d.file_ref.clone()));
-            }
-        }
-
-        self.deltas.insert(0, delta);
-        self.deltas.retain(|delta| delta.serial >= last_delta);
-    }
-
     pub fn create(session: RrdpSession, snapshot: SnapshotRef) -> Self {
-        Notification::new(session, 0, snapshot, vec![])
+        Notification::new(session, RRDP_FIRST_SERIAL, snapshot, vec![])
     }
 
     pub fn write_xml(&self, path: &Path) -> Result<(), KrillIoError> {
         trace!("Writing notification file: {}", path.to_string_lossy());
-        let mut file = file::create_file_with_path(&path)?;
+        let mut file = file::create_file_with_path(path)?;
 
         XmlWriter::encode_to_file(&mut file, |w| {
             let a = [
@@ -604,6 +585,20 @@ impl PublicationDeltaError {
         PublicationDeltaError::NoObjectForHashAndOrUri(uri.clone())
     }
 }
+
+//------------ RrdpFileRandom ------------------------------------------------
+#[derive(Clone, Debug, Deserialize, Eq, PartialEq, Serialize)]
+pub struct RrdpFileRandom(String);
+
+impl Default for RrdpFileRandom {
+    fn default() -> Self {
+        let mut bytes = [0; 8];
+        openssl::rand::rand_bytes(&mut bytes).unwrap();
+        let s = hex::encode(bytes);
+        RrdpFileRandom(s)
+    }
+}
+
 //------------ Snapshot ------------------------------------------------------
 
 /// A structure to contain the RRDP snapshot data.
@@ -611,6 +606,16 @@ impl PublicationDeltaError {
 pub struct Snapshot {
     session: RrdpSession,
     serial: u64,
+
+    // By using the default (i.e. random) for deserializing where this is absent, we do not
+    // need to migrate data when upgrading to this version where we introduce this new field.
+    // Note that this will result in new random values for existing Snapshot files, but
+    // because we now also perform a session reset on start up (see issue #533) it will
+    // not matter that this value does not map to the previous path of where this snapshot
+    // was stored, as it will be promptly replaced and forgotten.
+    #[serde(default)]
+    random: RrdpFileRandom,
+
     current_objects: CurrentObjects,
 }
 
@@ -619,6 +624,7 @@ impl Snapshot {
         Snapshot {
             session,
             serial,
+            random: RrdpFileRandom::default(),
             current_objects,
         }
     }
@@ -631,7 +637,8 @@ impl Snapshot {
         let current_objects = CurrentObjects::default();
         Snapshot {
             session,
-            serial: 0,
+            serial: RRDP_FIRST_SERIAL,
+            random: RrdpFileRandom::default(),
             current_objects,
         }
     }
@@ -639,7 +646,8 @@ impl Snapshot {
     pub fn session_reset(&self, session: RrdpSession) -> Self {
         Snapshot {
             session,
-            serial: 0,
+            serial: RRDP_FIRST_SERIAL,
+            random: RrdpFileRandom::default(),
             current_objects: self.current_objects.clone(),
         }
     }
@@ -654,6 +662,7 @@ impl Snapshot {
 
     pub fn apply_delta(&mut self, elements: DeltaElements, jail: &uri::Rsync) -> Result<(), PublicationDeltaError> {
         self.serial += 1;
+        self.random = RrdpFileRandom::default();
         self.current_objects.apply_delta(elements, jail)
     }
 
@@ -661,6 +670,18 @@ impl Snapshot {
         self.current_objects.elements().iter().fold(0, |sum, p| sum + p.size())
     }
 
+    fn rel_path(&self) -> String {
+        format!("{}/{}/{}/snapshot.xml", self.session, self.serial, self.random.0)
+    }
+
+    pub fn uri(&self, rrdp_base_uri: &uri::Https) -> uri::Https {
+        rrdp_base_uri.join(self.rel_path().as_ref()).unwrap()
+    }
+
+    pub fn path(&self, base_path: &Path) -> PathBuf {
+        base_path.join(self.rel_path())
+    }
+
     pub fn write_xml(&self, path: &Path) -> Result<(), KrillIoError> {
         trace!("Writing snapshot file: {}", path.to_string_lossy());
         let vec = self.xml();
@@ -765,6 +786,16 @@ impl From<publication::PublishDelta> for DeltaElements {
 pub struct Delta {
     session: RrdpSession,
     serial: u64,
+
+    // By using the default (i.e. random) for deserializing where this is absent, we do not
+    // need to migrate data when upgrading to this version where we introduce this new field.
+    // Note that this will result in new random values for existing Snapshot files, but
+    // because we now also perform a session reset on start up (see issue #533) it will
+    // not matter that this value does not map to the previous path of where this snapshot
+    // was stored, as it will be promptly replaced and forgotten.
+    #[serde(default)]
+    random: RrdpFileRandom,
+
     time: Time,
     elements: DeltaElements,
 }
@@ -774,6 +805,7 @@ impl Delta {
         Delta {
             session,
             time: Time::now(),
+            random: RrdpFileRandom::default(),
             serial,
             elements,
         }
@@ -820,11 +852,23 @@ impl Delta {
         (self.session, self.serial, self.elements)
     }
 
+    fn rel_path(&self) -> String {
+        format!("{}/{}/{}/delta.xml", self.session, self.serial, self.random.0)
+    }
+
+    pub fn uri(&self, rrdp_base_uri: &uri::Https) -> uri::Https {
+        rrdp_base_uri.join(self.rel_path().as_ref()).unwrap()
+    }
+
+    pub fn path(&self, base_path: &Path) -> PathBuf {
+        base_path.join(self.rel_path())
+    }
+
     pub fn write_xml(&self, path: &Path) -> Result<(), KrillIoError> {
         trace!("Writing delta file: {}", path.to_string_lossy());
         let vec = self.xml();
         let bytes = Bytes::from(vec);
-        file::save(&bytes, &path)?;
+        file::save(&bytes, path)?;
 
         Ok(())
     }
