@@ -1,22 +1,24 @@
-use std::fmt;
-use std::{collections::HashMap, path::Path};
+use std::{
+    collections::HashMap,
+    fmt,
+    path::Path,
+    str::FromStr,
+    sync::{Arc, RwLock},
+};
 
-use std::str::FromStr;
-use std::sync::{Arc, RwLock};
+use serde::{de::DeserializeOwned, Deserialize, Serialize};
 
-use serde::de::DeserializeOwned;
-use serde::{Deserialize, Serialize};
+use rpki::repository::x509::Time;
 
-use rpki::x509::Time;
-
-use crate::commons::eventsourcing::cmd::{Command, StoredCommandBuilder};
 use crate::commons::eventsourcing::{
+    cmd::{Command, StoredCommandBuilder},
     Aggregate, Event, KeyStoreKey, KeyValueError, KeyValueStore, PostSaveEventListener, StoredCommand,
     WithStorableDetails,
 };
 use crate::commons::{
     api::{CommandHistory, CommandHistoryCriteria, CommandHistoryRecord, Handle, Label},
     error::KrillIoError,
+    util::KrillVersion,
 };
 
 use super::PreSaveEventListener;
@@ -47,29 +49,6 @@ impl Default for StoredValueInfo {
     }
 }
 
-#[derive(Clone, Debug, Deserialize, Eq, Ord, PartialEq, PartialOrd, Serialize)]
-// Do NOT EVER change the order.. this is used to check whether migrations are needed
-// See this issue for planned fix: https://github.com/NLnetLabs/krill/issues/521
-#[allow(non_camel_case_types)]
-pub enum KeyStoreVersion {
-    Pre0_6,
-    V0_6,
-    V0_7,
-    V0_8_0_RC1,
-    V0_8,
-    V0_8_1_RC1,
-    V0_8_1,
-    V0_8_2,
-    V0_9_0_RC1,
-    V0_9_0,
-}
-
-impl KeyStoreVersion {
-    pub fn current() -> Self {
-        KeyStoreVersion::V0_9_0
-    }
-}
-
 //------------ CommandKey ----------------------------------------------------
 
 #[derive(Clone, Debug, Deserialize, Eq, PartialEq, Serialize)]
@@ -113,8 +92,8 @@ impl FromStr for CommandKey {
         if parts.len() != 4 || parts[0] != "command" {
             Err(CommandKeyError(s.to_string()))
         } else {
-            let timestamp_secs = i64::from_str(&parts[1]).map_err(|_| CommandKeyError(s.to_string()))?;
-            let sequence = u64::from_str(&parts[2]).map_err(|_| CommandKeyError(s.to_string()))?;
+            let timestamp_secs = i64::from_str(parts[1]).map_err(|_| CommandKeyError(s.to_string()))?;
+            let sequence = u64::from_str(parts[2]).map_err(|_| CommandKeyError(s.to_string()))?;
             // strip .json if present on the label part
             let label = {
                 let end = parts[3].to_string();
@@ -184,7 +163,7 @@ where
         };
 
         if !existed {
-            store.set_version(&KeyStoreVersion::current())?;
+            store.set_version(&KrillVersion::code_version())?;
         }
 
         Ok(store)
@@ -192,47 +171,67 @@ where
 
     /// Warms up the cache, to be used after startup. Will fail if any aggregates fail to load
     /// in which case a 'recover' operation can be tried.
+    pub fn warm(&self) -> StoreResult<()> {
+        for handle in self.list()? {
+            self.warm_aggregate(&handle)?;
+        }
+        info!("Cache for CAs has been warmed.");
+        Ok(())
+    }
+
+    /// Warm the cache for a specific aggregate. If successful save the latest snapshot
+    /// as well (will help in case of migrations where snapshots were dropped).
     ///
     /// In case any surplus event(s) and/or command(s) are encountered, i.e. extra entries not
     /// recorded in the 'info.json' which is always saved last on state changes - then it is
     /// assumed that an incomplete transaction took place. The surplus entries will be archived
     /// and warnings will be reported.
-    pub fn warm(&self) -> StoreResult<()> {
-        for handle in self.list()? {
-            let _ = self
-                .get_latest(&handle)
-                .map_err(|e| AggregateStoreError::WarmupFailed(handle.clone(), e.to_string()))?;
+    pub fn warm_aggregate(&self, handle: &Handle) -> StoreResult<()> {
+        info!("Warming the cache for CA: '{}'", handle);
 
-            // check that last command and event are consistent with
-            // the info, if not fail warmup and force recover
-            let info = self.get_info(&handle)?;
+        let agg = self
+            .get_latest(handle)
+            .map_err(|e| AggregateStoreError::WarmupFailed(handle.clone(), e.to_string()))?;
 
-            // for events we can just check if the next event, after
-            // the last event in the info exists
-            if self.get_event::<A::Event>(&handle, info.last_event + 1)?.is_some() {
-                warn!(
-                    "Found surplus event(s) for '{}' when warming up the cache. Will archive!",
-                    handle
-                );
-                self.archive_surplus_events(&handle, info.last_event + 1)?;
-            }
+        // check that last command and event are consistent with
+        // the info, if not fail warmup and force recover
+        let info = self.get_info(handle)?;
 
-            // Check if there are any commands with a sequence after the last
-            // recorded sequence in the info.
-            let mut crit = CommandHistoryCriteria::default();
-            crit.set_after_sequence(info.last_command);
-            let surplus_commands = self.command_keys_ascending(&handle, &crit)?;
-            if !surplus_commands.is_empty() {
-                warn!(
-                    "Found surplus command(s) for '{}' when warming up the cache. Will archive!",
-                    handle
-                );
+        // for events we can just check if the next event, after
+        // the last event in the info exists
+        if self.get_event::<A::Event>(handle, info.last_event + 1)?.is_some() {
+            warn!(
+                "Found surplus event(s) for '{}' when warming up the cache. Will archive!",
+                handle
+            );
+            self.archive_surplus_events(handle, info.last_event + 1)?;
+        }
 
-                for command in surplus_commands {
-                    self.archive_surplus_command(&handle, &command)?;
-                }
+        // Check if there are any commands with a sequence after the last
+        // recorded sequence in the info.
+        let mut crit = CommandHistoryCriteria::default();
+        crit.set_after_sequence(info.last_command);
+        let surplus_commands = self.command_keys_ascending(handle, &crit)?;
+        if !surplus_commands.is_empty() {
+            warn!(
+                "Found surplus command(s) for '{}' when warming up the cache. Will archive!",
+                handle
+            );
+
+            for command in surplus_commands {
+                self.archive_surplus_command(handle, &command)?;
             }
         }
+
+        // Save the snapshot if it does not yet match the latest state
+        if info.snapshot_version != agg.version() {
+            self.store_snapshot(handle, agg.as_ref())?;
+            debug!(
+                "Saved updated snapshot for '{}', to decrease future load times.",
+                handle
+            );
+        }
+
         Ok(())
     }
 
@@ -429,15 +428,16 @@ where
 
         let stored_command_builder = StoredCommandBuilder::new(&cmd, latest.version(), info.last_command);
 
-        let res = match latest.process_command(cmd) {
+        match latest.process_command(cmd) {
             Err(e) => {
                 let stored_command = stored_command_builder.finish_with_error(&e);
                 self.store_command(stored_command)?;
+                self.save_info(&handle, &info)?; // last_command was updated
                 Err(e)
             }
             Ok(events) => {
                 if events.is_empty() {
-                    return Ok(latest); // otherwise the version info will be updated
+                    Ok(latest) // note: no-op no version info will be updated
                 } else {
                     let agg = Arc::make_mut(&mut latest);
 
@@ -513,6 +513,9 @@ where
 
                     cache.insert(handle.clone(), Arc::new(agg.clone()));
 
+                    // Save the info so that the aggregate can be loaded properly by the listeners.
+                    self.save_info(&handle, &info)?;
+
                     // Now send the events to the 'post-save' listeners.
                     for listener in &self.post_save_listeners {
                         listener.as_ref().listen(agg, events.as_slice());
@@ -521,11 +524,7 @@ where
                     Ok(latest)
                 }
             }
-        };
-
-        self.save_info(&handle, &info)?;
-
-        res
+        }
     }
 
     /// Returns true if an instance exists for the id
@@ -647,15 +646,8 @@ where
     fn get_latest_no_lock(&self, handle: &Handle) -> StoreResult<Arc<A>> {
         trace!("Trying to load aggregate id: {}", handle);
 
-        let info_key = Self::key_for_info(handle);
-        let limit = self
-            .kv
-            .get::<StoredValueInfo>(&info_key)
-            .map_err(|_| AggregateStoreError::InfoCorrupt(handle.clone()))?
-            .map(|info| info.last_event);
-
         match self.cache_get(handle) {
-            None => match self.get_aggregate(handle, limit)? {
+            None => match self.get_aggregate(handle, None)? {
                 None => {
                     error!("Could not load aggregate with id: {} from disk", handle);
                     Err(AggregateStoreError::UnknownAggregate(handle.clone()))
@@ -670,7 +662,7 @@ where
             Some(mut arc) => {
                 if self.has_updates(handle, &arc)? {
                     let agg = Arc::make_mut(&mut arc);
-                    self.update_aggregate(handle, agg, limit)?;
+                    self.update_aggregate(handle, agg, None)?;
                 }
                 trace!("Loaded aggregate id: {} from memory", handle);
                 Ok(arc)
@@ -713,14 +705,14 @@ where
         KeyStoreKey::scoped(agg.to_string(), format!("{}.json", command))
     }
 
-    pub fn get_version(&self) -> Result<KeyStoreVersion, AggregateStoreError> {
-        match self.kv.get::<KeyStoreVersion>(&Self::key_version())? {
+    pub fn get_version(&self) -> Result<KrillVersion, AggregateStoreError> {
+        match self.kv.get::<KrillVersion>(&Self::key_version())? {
             Some(version) => Ok(version),
-            None => Ok(KeyStoreVersion::Pre0_6),
+            None => Ok(KrillVersion::v0_5_0_or_before()),
         }
     }
 
-    pub fn set_version(&self, version: &KeyStoreVersion) -> Result<(), AggregateStoreError> {
+    pub fn set_version(&self, version: &KrillVersion) -> Result<(), AggregateStoreError> {
         self.kv.store(&Self::key_version(), version)?;
         Ok(())
     }
@@ -844,7 +836,7 @@ where
                     if limit >= agg.version() - 1 {
                         aggregate_opt = Some(agg)
                     } else {
-                        trace!("Discarding snapshot after limit '{}'", id);
+                        warn!("Snapshot for '{}' is after version '{}', archiving it", id, limit);
                         self.kv.archive_surplus(&snapshot_key)?;
                     }
                 } else {
@@ -856,7 +848,7 @@ where
         }
 
         if aggregate_opt.is_none() {
-            warn!("No snapshot found for '{}' will try backup snapshot", id);
+            warn!("No suitable snapshot found for '{}' will try backup snapshot", id);
             let backup_snapshot_key = Self::key_for_backup_snapshot(id);
             match self.kv.get::<A>(&backup_snapshot_key) {
                 Err(e) => {
@@ -873,7 +865,10 @@ where
                         if limit >= agg.version() - 1 {
                             aggregate_opt = Some(agg)
                         } else {
-                            trace!("Discarding backup snapshot after limit '{}'", id);
+                            warn!(
+                                "Backup snapshot for '{}' is after version '{}', archiving it",
+                                id, limit
+                            );
                             self.kv.archive_surplus(&backup_snapshot_key)?;
                         }
                     } else {
@@ -886,7 +881,10 @@ where
         }
 
         if aggregate_opt.is_none() {
-            warn!("No snapshots found for '{}' will try from initialization event.", id);
+            warn!(
+                "No suitable snapshot for '{}' will rebuild state from events. This can take some time.",
+                id
+            );
             let init_key = Self::key_for_event(id, 0);
             aggregate_opt = match self.kv.get::<A::InitEvent>(&init_key)? {
                 Some(e) => {
@@ -908,59 +906,46 @@ where
 
     fn update_aggregate(&self, id: &Handle, aggregate: &mut A, limit: Option<u64>) -> Result<(), AggregateStoreError> {
         let start = aggregate.version();
-        let limit = if let Some(limit) = limit {
-            debug!("Will attempt to update '{}' using explicit limit", id);
-            limit
-        } else if let Ok(info) = self.get_info(id) {
-            debug!("Will attempt to update '{}' using limit from info", id);
-            info.last_event
+
+        if let Some(limit) = limit {
+            debug!("Will update '{}' from version: {} to: {}", id, start, limit + 1);
         } else {
-            let nr_events = self.kv.keys(Some(id.to_string()), "delta-")?.len();
-            if nr_events < 1 {
-                return Err(AggregateStoreError::InfoMissing(id.clone()));
-            } else {
-                let limit = (nr_events - 1) as u64;
-                debug!("Will attempt to update '{}' from limit based on nr of events", id,);
-                limit
+            debug!("Will update '{}' to latest version", id);
+        }
+
+        // check and apply any applicable events until:
+        // - the limit is reached (if supplied)
+        // - there are no more events
+        // - any event cannot be applied (return an error)
+        loop {
+            let version = aggregate.version();
+            if let Some(limit) = limit {
+                if limit == version - 1 {
+                    debug!("Updated '{}' to: {}", id, version);
+                    break;
+                }
             }
-        };
 
-        if limit == aggregate.version() - 1 {
-            // already at version, done
-            // note that an event has the version of the aggregate it *affects*. So delta 10 results in version 11.
-            debug!("Snapshot for '{}' is up to date", id);
-            return Ok(());
-        }
-
-        debug!(
-            "Will attempt to update '{}' from version: {} to: {}",
-            id,
-            start,
-            limit + 1
-        );
-
-        if start > limit {
-            return Err(AggregateStoreError::ReplayError(id.clone(), limit, start));
-        }
-
-        for version in start..limit + 1 {
-            if let Some(e) = self.get_event(id, version)? {
-                if aggregate.version() != version {
+            if let Some(e) = self.get_event::<A::Event>(id, version)? {
+                if version != e.version() {
                     error!("Trying to apply event to wrong version of aggregate in replay");
-                    return Err(AggregateStoreError::ReplayError(id.clone(), limit, version));
+                    return Err(AggregateStoreError::ReplayError(id.clone(), version, e.version()));
                 }
                 aggregate.apply(e);
                 debug!("Applied event nr {} to aggregate {}", version, id);
             } else {
-                return Err(AggregateStoreError::ReplayError(id.clone(), limit, version));
+                debug!("No more events found. updated '{}' to: {}", id, version);
+                break;
             }
         }
 
         Ok(())
     }
 
-    /// Saves the latest snapshot - overwrites any previous snapshot.
-    fn store_snapshot<V: Aggregate>(&self, id: &Handle, aggregate: &V) -> Result<(), AggregateStoreError> {
+    /// Saves the latest snapshot - backs up previous snapshot, and drops previous backup.
+    /// Uses moves to ensure that files are written entirely before they are made available
+    /// for reading.
+    pub fn store_snapshot<V: Aggregate>(&self, id: &Handle, aggregate: &V) -> Result<(), AggregateStoreError> {
         let snapshot_new = Self::key_for_new_snapshot(id);
         let snapshot_current = Self::key_for_snapshot(id);
         let snapshot_backup = Self::key_for_backup_snapshot(id);
@@ -1035,10 +1020,10 @@ impl fmt::Display for AggregateStoreError {
             AggregateStoreError::InitError(handle) => {
                 write!(f, "Init event exists for '{}', but cannot be applied", handle)
             }
-            AggregateStoreError::ReplayError(handle, target_version, fail_version) => write!(
+            AggregateStoreError::ReplayError(handle, version, fail_version) => write!(
                 f,
-                "Cannot reconstruct '{}' to version '{}', failed at version {}",
-                handle, target_version, fail_version
+                "Event for '{}' version '{}' had version '{}'",
+                handle, version, fail_version
             ),
             AggregateStoreError::InfoMissing(handle) => write!(f, "Missing stored value info for '{}'", handle),
             AggregateStoreError::InfoCorrupt(handle) => write!(f, "Corrupt stored value info for '{}'", handle),
