@@ -1,23 +1,27 @@
-use std::str::FromStr;
-use std::{convert::TryFrom, fmt::Display};
-use std::{fmt, io};
+use std::{convert::TryFrom, fmt, fmt::Display, io, str::FromStr};
 
 use bytes::Bytes;
 use chrono::{DateTime, SecondsFormat, Utc};
 
-use rpki::cert::Cert;
-use rpki::crypto::KeyIdentifier;
-use rpki::csr::Csr;
-use rpki::resources::{AsBlocks, IpBlocks};
-use rpki::uri;
-use rpki::x509::Time;
+use rpki::{
+    repository::{
+        cert::Cert,
+        crypto::KeyIdentifier,
+        csr::Csr,
+        resources::{AsBlocks, IpBlocks},
+        x509::Time,
+    },
+    uri,
+};
 
-use crate::commons::api::{
-    EntitlementClass, Entitlements, Handle, IssuanceRequest, IssuanceResponse, IssuedCert, RequestResourceLimit,
-    ResourceClassName, ResourceSet, ResourceSetError, RevocationRequest, RevocationResponse, SigningCert,
+use crate::commons::{
+    api::{
+        EntitlementClass, Entitlements, Handle, IssuanceRequest, IssuanceResponse, IssuedCert, RequestResourceLimit,
+        ResourceClassName, ResourceSet, ResourceSetError, RevocationRequest, RevocationResponse, SigningCert,
+    },
+    crypto::ProtocolCms,
+    util::xml::{AttributesError, XmlReader, XmlReaderErr, XmlWriter},
 };
-use crate::commons::crypto::ProtocolCms;
-use crate::commons::util::xml::{AttributesError, XmlReader, XmlReaderErr, XmlWriter};
 
 //------------ Consts --------------------------------------------------------
 
@@ -69,6 +73,10 @@ impl Message {
     pub fn content(&self) -> &Content {
         &self.content
     }
+
+    pub fn is_list_response(&self) -> bool {
+        matches!(&self.content, Content::Res(Res::List(_)))
+    }
 }
 
 /// # Convenience accessors
@@ -729,6 +737,10 @@ impl NotPerformedResponse {
         }
     }
 
+    pub fn status(&self) -> u64 {
+        self.status
+    }
+
     /// Creates a response for a status value defined in RFC6492. Also adds
     /// the description defined in the RFC.
     pub fn from_code(code: &str) -> Result<Self, Error> {
