@@ -3,24 +3,28 @@
 //! Support for the RFC8183 out-of-band setup requests and responses
 //! used to exchange identity and configuration between CAs and their
 //! parent CA and/or RPKI Publication Servers.
-use std::str::{from_utf8_unchecked, FromStr};
-use std::{convert::TryFrom, path::Path};
-use std::{fmt, io};
+use std::{
+    convert::TryFrom,
+    fmt, io,
+    path::Path,
+    str::{from_utf8_unchecked, FromStr},
+};
 
 use base64::DecodeError;
 use bcder::decode;
 use bytes::Bytes;
 use serde::{de, Deserialize, Deserializer, Serialize, Serializer};
 
-use rpki::uri;
-use rpki::x509;
+use rpki::{repository::x509, uri};
 
-use crate::commons::crypto::IdCert;
-use crate::commons::util::file;
-use crate::commons::util::xml::{AttributesError, XmlReader, XmlReaderErr, XmlWriter};
 use crate::commons::{
-    api::{Handle, PublisherHandle, RepoInfo},
+    api::{Handle, ParentHandle, PublisherHandle, RepoInfo},
+    crypto::IdCert,
     error::KrillIoError,
+    util::{
+        file,
+        xml::{AttributesError, XmlReader, XmlReaderErr, XmlWriter},
+    },
 };
 
 pub const VERSION: &str = "1";
@@ -681,13 +685,36 @@ impl fmt::Display for RepositoryResponse {
 }
 //------------ ServiceUri ----------------------------------------------------
 
-/// The service URI where a child or publisher needs to send its
+/// The service URI where a child or publisher needs to send RFC 6492 or 8181 messages
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub enum ServiceUri {
     Https(uri::Https),
     Http(String),
 }
 
+impl ServiceUri {
+    /// Returns the handle of the local parent for this specific ServiceUri, and the
+    /// configured base (service) uri. Provided that this indeed maps back to this
+    /// same server and it is an rfc6492 style krill uri.
+    pub fn local_parent(&self, base_uri: &uri::Https) -> Option<ParentHandle> {
+        match &self {
+            Self::Http(_) => None,
+            Self::Https(service_uri) => {
+                let service_uri = service_uri.as_str();
+                let base_uri = base_uri.as_str();
+
+                if let Some(path) = service_uri.strip_prefix(base_uri) {
+                    if let Some(ca_name) = path.strip_prefix("rfc6492/") {
+                        return ParentHandle::from_str(ca_name).ok();
+                    }
+                }
+
+                None
+            }
+        }
+    }
+}
+
 impl TryFrom<String> for ServiceUri {
     type Error = Error;
 
@@ -799,7 +826,7 @@ impl From<uri::Error> for Error {
 
 #[cfg(test)]
 mod tests {
-    use rpki::x509::Time;
+    use rpki::repository::x509::Time;
 
     use crate::test;
     use crate::test::test_id_certificate;
@@ -950,4 +977,13 @@ mod tests {
         let xml = include_str!("../../../test-resources/oob/apnic/repository.response.xml");
         RepositoryResponse::validate_at(xml.as_bytes(), apnic_oob_time()).unwrap();
     }
+
+    #[test]
+    fn derive_local_parent_from_service_uri() {
+        let service_uri = ServiceUri::Https(uri::Https::from_str("https://localhost/rfc6492/PARENT-ID").unwrap());
+        let base_uri = uri::Https::from_str("https://localhost/").unwrap();
+
+        let parent = service_uri.local_parent(&base_uri).unwrap();
+        assert_eq!(parent, ParentHandle::from_str("PARENT-ID").unwrap())
+    }
 }
