@@ -4,11 +4,10 @@ use std::{
     time::{Duration, SystemTime, UNIX_EPOCH},
 };
 
-use crate::commons::api::Token;
-use crate::commons::error::Error;
-use crate::commons::KrillResult;
-
-use super::crypt::{self, CryptState, NonceState};
+use crate::{
+    commons::{api::Token, error::Error, KrillResult},
+    daemon::auth::common::crypt::{self, CryptState, NonceState},
+};
 
 const MAX_CACHE_SECS: u64 = 30;
 
@@ -144,7 +143,7 @@ impl LoginSessionCache {
     fn lookup_session(&self, token: &Token) -> Option<ClientSession> {
         match self.cache.read() {
             Ok(readable_cache) => {
-                if let Some(cache_item) = readable_cache.get(&token) {
+                if let Some(cache_item) = readable_cache.get(token) {
                     return Some(cache_item.session.clone());
                 }
             }
@@ -256,18 +255,20 @@ impl LoginSessionCache {
             .map_err(|err| Error::Custom(format!("Unable to purge session cache: {}", err)))?;
 
         let size_before = cache.len();
-        let now = Self::time_now_secs_since_epoch()?;
 
         // Only retain cache items that have been cached for less than the
         // maximum time allowed.
+        let now = Self::time_now_secs_since_epoch()?;
         cache.retain(|_, v| v.evict_after > now);
 
-        let size_after = size_before - cache.len();
+        let size_after = cache.len();
 
-        debug!(
-            "Login session cache purge: size before={}, size after={}",
-            size_before, size_after
-        );
+        if size_after != size_before {
+            debug!(
+                "Login session cache purge: size before={}, size after={}",
+                size_before, size_after
+            );
+        }
 
         Ok(())
     }
