@@ -27,51 +27,63 @@ use std::{
     },
     ops::Deref,
     path::Path,
-    sync::{Arc, RwLock, RwLockReadGuard},
+    sync::Arc,
 };
 
+use tokio::sync::{RwLock, RwLockReadGuard};
+
 use basic_cookies::Cookie;
 use hyper::header::{HeaderValue, SET_COOKIE};
 use jmespatch as jmespath;
 use jmespath::ToJmespath;
 
-use openidconnect::UserInfoError;
-use openidconnect::{core::CoreRevocableToken, AccessToken, RequestTokenError, RevocationErrorResponseType};
 use openidconnect::{
     core::{
         CoreAuthPrompt, CoreErrorResponseType, CoreIdTokenVerifier, CoreJwsSigningAlgorithm, CoreResponseMode,
-        CoreResponseType,
+        CoreResponseType, CoreRevocableToken,
     },
-    RevocationUrl,
-};
-use openidconnect::{
-    AuthenticationFlow, AuthorizationCode, ClientId, ClientSecret, CsrfToken, IssuerUrl, Nonce, OAuth2TokenResponse,
-    RedirectUrl, RefreshToken, Scope,
+    AccessToken, AuthenticationFlow, AuthorizationCode, ClientId, ClientSecret, CsrfToken, IssuerUrl, Nonce,
+    OAuth2TokenResponse, RedirectUrl, RefreshToken, RequestTokenError, RevocationErrorResponseType, RevocationUrl,
+    Scope, UserInfoError,
 };
 
 use urlparse::{urlparse, GetQuery};
 
-use crate::commons::util::sha256;
-use crate::commons::KrillResult;
-use crate::commons::{actor::ActorDef, api::Token};
-use crate::daemon::auth::common::crypt;
-use crate::daemon::auth::common::session::*;
-use crate::daemon::auth::providers::config_file::config::ConfigUserDetails;
-use crate::daemon::auth::providers::openid_connect::config::ConfigAuthOpenIDConnectClaims;
-use crate::daemon::auth::providers::openid_connect::httpclient::logging_http_client;
-use crate::daemon::auth::providers::openid_connect::jmespathext;
-use crate::daemon::auth::{Auth, AuthProvider, LoggedInUser};
-use crate::daemon::config::Config;
-use crate::daemon::http::auth::url_encode;
-use crate::daemon::http::auth::AUTH_CALLBACK_ENDPOINT;
-use crate::daemon::http::HttpResponse;
-use crate::{commons::error::Error, daemon::auth::common::crypt::CryptState};
-
-use super::config::{
-    ConfigAuthOpenIDConnect, ConfigAuthOpenIDConnectClaim, ConfigAuthOpenIDConnectClaimSource as ClaimSource,
-};
-use super::util::{
-    FlexibleClient, FlexibleIdTokenClaims, FlexibleTokenResponse, FlexibleUserInfoClaims, LogOrFail, WantedMeta,
+use crate::{
+    commons::{
+        actor::ActorDef,
+        api::Token,
+        error::Error,
+        util::{httpclient, sha256},
+        KrillResult,
+    },
+    daemon::{
+        auth::{
+            common::{
+                crypt::{self, CryptState},
+                session::*,
+            },
+            providers::config_file::config::ConfigUserDetails,
+            providers::openid_connect::{
+                config::{
+                    ConfigAuthOpenIDConnect, ConfigAuthOpenIDConnectClaim,
+                    ConfigAuthOpenIDConnectClaimSource as ClaimSource, ConfigAuthOpenIDConnectClaims,
+                },
+                httpclient::logging_http_client,
+                jmespathext,
+                util::{
+                    FlexibleClient, FlexibleIdTokenClaims, FlexibleTokenResponse, FlexibleUserInfoClaims, LogOrFail,
+                    WantedMeta,
+                },
+            },
+            Auth, LoggedInUser,
+        },
+        config::Config,
+        http::{
+            auth::{url_encode, AUTH_CALLBACK_ENDPOINT},
+            HttpResponse,
+        },
+    },
 };
 
 // On modern browsers (Chrome >= 51, Edge >= 16, Firefox >= 60 & Safari >= 12) the "__Host" prefix is a defence-in-depth
@@ -151,19 +163,19 @@ impl OpenIDConnectAuthProvider {
         })
     }
 
-    fn initialize_connection_if_needed(&self) -> KrillResult<()> {
-        let mut conn_guard = self.conn.write().unwrap(); // should never fail, better to panic and crash out if it does
+    async fn initialize_connection_if_needed(&self) -> KrillResult<()> {
+        let mut conn_guard = self.conn.write().await;
 
         if conn_guard.is_none() {
-            *conn_guard = Some(self.initialize_connection()?);
+            *conn_guard = Some(self.initialize_connection().await?);
         }
 
         Ok(())
     }
 
-    fn initialize_connection(&self) -> KrillResult<ProviderConnectionProperties> {
+    async fn initialize_connection(&self) -> KrillResult<ProviderConnectionProperties> {
         trace!("OpenID Connect: Initializing provider connection...");
-        let meta = self.discover()?;
+        let meta = self.discover().await?;
         let (email_scope_supported, userinfo_endpoint_supported, logout_mode) =
             self.check_provider_capabilities(&meta)?;
         let client = self.build_client(meta, &logout_mode)?;
@@ -182,7 +194,7 @@ impl OpenIDConnectAuthProvider {
     /// discovery endpoint of the provider, e.g.
     ///   https://<provider.domain>/<something/.well-known/openid-configuration
     /// Via which we can discover both endpoint URIs and capability flags.
-    fn discover(&self) -> KrillResult<WantedMeta> {
+    async fn discover(&self) -> KrillResult<WantedMeta> {
         // Read from config the OpenID Connect identity provider discovery URL.
         // Strip off /.well-known/openid-configuration because the openid-connect
         // crate wants to add this itself and will fail if it is already present
@@ -198,13 +210,15 @@ impl OpenIDConnectAuthProvider {
 
         // Contact the OpenID Connect: identity provider discovery endpoint to
         // learn about and configure ourselves to talk to it.
-        let meta = WantedMeta::discover(&issuer, logging_http_client).map_err(|e| {
-            Error::custom(format!(
-                "OpenID Connect: Discovery failed with issuer {}, {}",
-                issuer.as_str(),
-                stringify_cause_chain(e)
-            ))
-        })?;
+        let meta = WantedMeta::discover_async(issuer.clone(), logging_http_client)
+            .await
+            .map_err(|e| {
+                Error::custom(format!(
+                    "OpenID Connect: Discovery failed with issuer {}, {}",
+                    issuer.as_str(),
+                    stringify_cause_chain(e)
+                ))
+            })?;
 
         Ok(meta)
     }
@@ -397,6 +411,7 @@ impl OpenIDConnectAuthProvider {
             self.config
                 .service_uri()
                 .join(AUTH_CALLBACK_ENDPOINT.trim_start_matches('/').as_bytes())
+                .unwrap()
                 .to_string(),
         )?;
 
@@ -439,7 +454,7 @@ impl OpenIDConnectAuthProvider {
         ))
     }
 
-    fn try_revoke_token(&self, session: &ClientSession) -> Result<(), RevocationErrorResponseType> {
+    async fn try_revoke_token(&self, session: &ClientSession) -> Result<(), RevocationErrorResponseType> {
         // Connect to the OpenID Connect provider OAuth 2.0 token revocation endpoint to terminate the
         // provider session
         // From: https://tools.ietf.org/html/rfc7009#section-2
@@ -459,6 +474,7 @@ impl OpenIDConnectAuthProvider {
         trace!("OpenID Connect: Submitting RFC-7009 section 2 Token Revocation request");
         let lock_guard = self
             .get_connection()
+            .await
             .map_err(|err| RevocationErrorResponseType::Basic(CoreErrorResponseType::Extension(err.to_string())))?;
         let conn = lock_guard.deref().as_ref().unwrap(); // safe to unwrap as was tested in get_connection()
 
@@ -471,7 +487,8 @@ impl OpenIDConnectAuthProvider {
                     err.to_string()
                 )))
             })?
-            .request(logging_http_client)
+            .request_async(logging_http_client)
+            .await
         {
             Ok(_) => Ok(()),
             Err(err) => match &err {
@@ -514,7 +531,7 @@ impl OpenIDConnectAuthProvider {
     /// Try refreshing the token once with the OIDC Provider and return either the new token, or the Error received from
     /// the OpenID Connect Provider. This Error is FOR INTERNAL CONSUMPTION only. The caller of this function is
     /// responsible for creating end-user error messages, logging and (optionally) retrying.
-    fn try_refresh_token(&self, session: &ClientSession) -> Result<Auth, CoreErrorResponseType> {
+    async fn try_refresh_token(&self, session: &ClientSession) -> Result<Auth, CoreErrorResponseType> {
         let refresh_token = &session.secrets.get(TokenKind::RefreshToken.into()).ok_or_else(|| {
             CoreErrorResponseType::Extension(
                 "Internal error: Token refresh attempted without a refresh token".to_string(),
@@ -526,13 +543,15 @@ impl OpenIDConnectAuthProvider {
 
         let lock_guard = self
             .get_connection()
+            .await
             .map_err(|err| CoreErrorResponseType::Extension(err.to_string()))?;
         let conn = lock_guard.deref().as_ref().unwrap(); // safe to unwrap as was tested in get_connection()
 
         let token_response = conn
             .client
             .exchange_refresh_token(&RefreshToken::new(refresh_token.to_string()))
-            .request(logging_http_client);
+            .request_async(logging_http_client)
+            .await;
 
         match token_response {
             Ok(token_response) => {
@@ -809,8 +828,8 @@ impl OpenIDConnectAuthProvider {
         None
     }
 
-    fn get_connection(&self) -> KrillResult<RwLockReadGuard<Option<ProviderConnectionProperties>>> {
-        let conn_guard = self.conn.read().unwrap(); // should never fail, better to panic and crash out if it does
+    async fn get_connection<'a>(&'_ self) -> KrillResult<RwLockReadGuard<'_, Option<ProviderConnectionProperties>>> {
+        let conn_guard = self.conn.read().await;
 
         conn_guard.as_ref().ok_or_else(|| {
             OpenIDConnectAuthProvider::internal_error("Connection to provider not yet established", None)
@@ -855,14 +874,15 @@ impl OpenIDConnectAuthProvider {
         }
     }
 
-    fn get_token_response(&self, code: Token) -> KrillResult<FlexibleTokenResponse> {
-        let lock_guard = self.get_connection()?;
+    async fn get_token_response(&self, code: Token) -> KrillResult<FlexibleTokenResponse> {
+        let lock_guard = self.get_connection().await?;
         let conn = lock_guard.deref().as_ref().unwrap(); // safe to unwrap as was tested in get_connection()
 
         let token_response: FlexibleTokenResponse = conn
             .client
             .exchange_code(AuthorizationCode::new(code.to_string()))
-            .request(logging_http_client)
+            .request_async(logging_http_client)
+            .await
             .map_err(|e| {
                 let (msg, additional_info) = match e {
                     RequestTokenError::ServerResponse(ref provider_err) => {
@@ -903,12 +923,13 @@ impl OpenIDConnectAuthProvider {
         Ok(token_response)
     }
 
-    fn get_token_id_claims<'a>(
+    #[allow(clippy::needless_lifetimes)] // clippy says it can be elided, but.. it seems like it's a false positive.
+    async fn get_token_id_claims<'a>(
         &self,
         token_response: &'a FlexibleTokenResponse,
         nonce_hash: Nonce,
     ) -> KrillResult<&'a FlexibleIdTokenClaims> {
-        let lock_guard = self.get_connection()?;
+        let lock_guard = self.get_connection().await?;
         let conn = lock_guard.deref().as_ref().unwrap(); // safe to unwrap as was tested in get_connection()
         let mut id_token_verifier: CoreIdTokenVerifier = conn.client.id_token_verifier();
 
@@ -945,11 +966,11 @@ impl OpenIDConnectAuthProvider {
         Ok(id_token_claims)
     }
 
-    fn get_user_info_claims(
+    async fn get_user_info_claims(
         &self,
         token_response: &FlexibleTokenResponse,
     ) -> KrillResult<Option<FlexibleUserInfoClaims>> {
-        let lock_guard = self.get_connection()?;
+        let lock_guard = self.get_connection().await?;
         let conn = lock_guard.deref().as_ref().unwrap(); // safe to unwrap as was tested in get_connection()
 
         let user_info_claims: Option<FlexibleUserInfoClaims> = if conn.userinfo_endpoint_supported {
@@ -969,7 +990,8 @@ impl OpenIDConnectAuthProvider {
                     // don't require the response to be signed as the spec says
                     // signing it is optional: See: https://openid.net/specs/openid-connect-core-1_0.html#UserInfoResponse
                     .require_signed_response(false)
-                    .request(logging_http_client)
+                    .request_async(logging_http_client)
+                    .await
                     .map_err(|e| {
                         let msg = match e {
                             UserInfoError::ClaimsVerification(ref provider_err) => {
@@ -1026,7 +1048,7 @@ impl OpenIDConnectAuthProvider {
                     // Lookup the claim value in the auth_users config file section
                     user.unwrap().attributes.get(&attr_name.to_string()).cloned()
                 }
-                _ => self.extract_claim(&claim_conf, &id_token_claims, user_info_claims.as_ref())?,
+                _ => self.extract_claim(&claim_conf, id_token_claims, user_info_claims.as_ref())?,
             };
 
             if let Some(attr_value) = attr_value {
@@ -1077,7 +1099,7 @@ impl OpenIDConnectAuthProvider {
     }
 }
 
-impl AuthProvider for OpenIDConnectAuthProvider {
+impl OpenIDConnectAuthProvider {
     // Connect Core 1.0 section 3.1.26 Authentication Error Response
     // OAuth 2.0 RFC-674 4.1.2.1 (Authorization Request Errors) & 5.2 (Access Token Request Errors)
 
@@ -1086,17 +1108,17 @@ impl AuthProvider for OpenIDConnectAuthProvider {
     /// an error to report back to the user (one of the ApiAuth* Error types).
     /// Make sure to not leak any OIDC implementation details into the Error result!
     /// This function is also responsible for all logging around refreshing the token / extending the session.
-    fn authenticate(&self, request: &hyper::Request<hyper::Body>) -> KrillResult<Option<ActorDef>> {
+    pub async fn authenticate(&self, request: &hyper::Request<hyper::Body>) -> KrillResult<Option<ActorDef>> {
         trace!("Attempting to authenticate the request..");
 
-        self.initialize_connection_if_needed().map_err(|err| {
+        self.initialize_connection_if_needed().await.map_err(|err| {
             OpenIDConnectAuthProvider::internal_error(
                 "OpenID Connect: Cannot authenticate request: Failed to connect to provider",
                 Some(&stringify_cause_chain(err)),
             )
         })?;
 
-        let res = match self.get_bearer_token(request) {
+        let res = match httpclient::get_bearer_token(request) {
             Some(token) => {
                 // see if we can decode, decrypt and deserialize the users token
                 // into a login session structure
@@ -1125,7 +1147,7 @@ impl AuthProvider for OpenIDConnectAuthProvider {
                 }
 
                 // Token needs refresh and we have a refresh token, try to refresh
-                let new_auth = match self.try_refresh_token(&session) {
+                let new_auth = match self.try_refresh_token(&session).await {
                     Ok(auth) => {
                         trace!(
                             "OpenID Connect: Successfully refreshed token for user \"{}\"",
@@ -1206,7 +1228,7 @@ impl AuthProvider for OpenIDConnectAuthProvider {
     /// URL should be requested by the client on every login as the intention is
     /// that it contains randomly generated CSFF token and nonce values which
     /// can be used to protect against certain cross-site and replay attacks.
-    fn get_login_url(&self) -> KrillResult<HttpResponse> {
+    pub async fn get_login_url(&self) -> KrillResult<HttpResponse> {
         // TODO: we probably should do some more work here to ensure we get the
         // proper security benefits of the CSRF token, currently we are
         // discarding the CSRF token instead of checking it.
@@ -1225,7 +1247,7 @@ impl AuthProvider for OpenIDConnectAuthProvider {
         //    hash of the session cookie to detect ID Token replay by third
         //    parties"
 
-        self.initialize_connection_if_needed().map_err(|err| {
+        self.initialize_connection_if_needed().await.map_err(|err| {
             OpenIDConnectAuthProvider::internal_error(
                 "OpenID Connect: Cannot get login URL: Failed to connect to provider",
                 Some(&stringify_cause_chain(err)),
@@ -1239,7 +1261,7 @@ impl AuthProvider for OpenIDConnectAuthProvider {
         let nonce_b64_str = random_value.secret();
         let nonce_hash = sha256(nonce_b64_str.as_bytes());
 
-        let lock_guard = self.get_connection()?;
+        let lock_guard = self.get_connection().await?;
         let conn = lock_guard.deref().as_ref().unwrap(); // safe to unwrap as was tested in get_connection()
 
         // At the time of writing the underlying oauth2 crate CsrfToken::new_random() function is used to
@@ -1303,6 +1325,10 @@ impl AuthProvider for OpenIDConnectAuthProvider {
             || Nonce::new(base64::encode_config(nonce_hash, base64::URL_SAFE_NO_PAD)),
         );
 
+        // This unwrap is safe as we check in new() that the OpenID Connect
+        // config exists.
+        let oidc_conf = self.oidc_conf()?;
+
         // From https://openid.net/specs/openid-connect-core-1_0.html#AuthRequest:
         //   "prompt: login - The Authorization Server SHOULD prompt the
         //    End-User for re-authentication. If it cannot re-authenticate the
@@ -1312,7 +1338,11 @@ impl AuthProvider for OpenIDConnectAuthProvider {
         // to specify who to login as, we don't want the provider somehow
         // automatically completing the login process because it has some notion
         // of an existing login session.
-        request = request.add_prompt(CoreAuthPrompt::Login);
+
+        // https://github.com/NLnetLabs/krill/issues/614
+        if oidc_conf.prompt_for_login {
+            request = request.add_prompt(CoreAuthPrompt::Login);
+        }
 
         // The "openid" scope that OpenID Connect: providers are required to
         // check for is sent automatically by the openidconnect crate. We can
@@ -1332,10 +1362,6 @@ impl AuthProvider for OpenIDConnectAuthProvider {
 
         // TODO: use request.set_pkce_challenge() ?
 
-        // This unwrap is safe as we check in new() that the OpenID Connect
-        // config exists.
-        let oidc_conf = self.oidc_conf()?;
-
         for scope in &oidc_conf.extra_login_scopes {
             request = request.add_scope(Scope::new(scope.clone()));
         }
@@ -1397,8 +1423,8 @@ impl AuthProvider for OpenIDConnectAuthProvider {
         Ok(HttpResponse::new(res))
     }
 
-    fn login(&self, request: &hyper::Request<hyper::Body>) -> KrillResult<LoggedInUser> {
-        self.initialize_connection_if_needed().map_err(|err| {
+    pub async fn login(&self, request: &hyper::Request<hyper::Body>) -> KrillResult<LoggedInUser> {
+        self.initialize_connection_if_needed().await.map_err(|err| {
             OpenIDConnectAuthProvider::internal_error(
                 "OpenID Connect: Cannot login user: Failed to connect to provider",
                 Some(&stringify_cause_chain(err)),
@@ -1429,7 +1455,7 @@ impl AuthProvider for OpenIDConnectAuthProvider {
                 // ==========================================================================================
                 trace!("OpenID Connect: Submitting RFC-6749 section 4.1.3 Access Token Request");
 
-                let token_response = self.get_token_response(code)?;
+                let token_response = self.get_token_response(code).await?;
 
                 // TODO: extract and keep the access token and refresh token so
                 // that we can extend the login session later. These are
@@ -1483,7 +1509,7 @@ impl AuthProvider for OpenIDConnectAuthProvider {
                 // https://openid.net/specs/openid-connect-core-1_0.html#NonceNotes
                 let nonce_hash = Nonce::new(base64::encode_config(sha256(nonce.as_bytes()), base64::URL_SAFE_NO_PAD));
 
-                let id_token_claims = self.get_token_id_claims(&token_response, nonce_hash)?;
+                let id_token_claims = self.get_token_id_claims(&token_response, nonce_hash).await?;
 
                 // TODO: There's also a suggestion to verify the access token
                 // received above using the at_hash claim in the ID token, if
@@ -1495,7 +1521,7 @@ impl AuthProvider for OpenIDConnectAuthProvider {
                 // See: https://openid.net/specs/openid-connect-core-1_0.html#UserInfo
                 // ==========================================================================================
 
-                let user_info_claims = self.get_user_info_claims(&token_response)?;
+                let user_info_claims = self.get_user_info_claims(&token_response).await?;
 
                 // ==========================================================================================
                 // Step 4: Extract and validate the "claims" that tells us which
@@ -1523,7 +1549,7 @@ impl AuthProvider for OpenIDConnectAuthProvider {
                 })?;
 
                 let id = self
-                    .extract_claim(&id_claim_conf, &id_token_claims, user_info_claims.as_ref())?
+                    .extract_claim(id_claim_conf, id_token_claims, user_info_claims.as_ref())?
                     .ok_or_else(|| OpenIDConnectAuthProvider::internal_error("No value found for 'id' claim", None))?;
 
                 // Lookup the a user in the config file authentication provider
@@ -1597,9 +1623,9 @@ impl AuthProvider for OpenIDConnectAuthProvider {
     /// logout page is not possible, instead from the end-user's perspective they are returned to the Lagosta web UI
     /// index page (which currently immediately redirects the user to the 3rd party OpenID Connect provider login page)
     /// but before that Krill contacts the provider on the logged-in users behalf to revoke their token at the provider.
-    fn logout(&self, request: &hyper::Request<hyper::Body>) -> KrillResult<HttpResponse> {
+    pub async fn logout(&self, request: &hyper::Request<hyper::Body>) -> KrillResult<HttpResponse> {
         // verify the bearer token indeed represents a logged-in Krill OpenID Connect provider session
-        let token = self.get_bearer_token(request).ok_or_else(|| {
+        let token = httpclient::get_bearer_token(request).ok_or_else(|| {
             warn!("Unexpectedly received a logout request without a session token.");
             Error::ApiInvalidCredentials("Invalid session token".to_string())
         })?;
@@ -1619,7 +1645,7 @@ impl AuthProvider for OpenIDConnectAuthProvider {
 
         // 2. verify that the provider is at least to some extent available, there's no point trying to log the token
         //    out of the provider if we know there's a problem with the provider
-        self.initialize_connection_if_needed().map_err(|err| {
+        self.initialize_connection_if_needed().await.map_err(|err| {
             OpenIDConnectAuthProvider::internal_error(
                 "OpenID Connect: Cannot logout with provider: Failed to connect to provider",
                 Some(&stringify_cause_chain(err)),
@@ -1627,7 +1653,7 @@ impl AuthProvider for OpenIDConnectAuthProvider {
         })?;
 
         // 3. use the provider connection details to contact the provider to terminate the client session
-        let lock_guard = self.get_connection()?;
+        let lock_guard = self.get_connection().await?;
         let conn = lock_guard.deref().as_ref().unwrap(); // safe to unwrap as was tested in get_connection()
 
         let go_to_url = match &conn.logout_mode {
@@ -1635,7 +1661,7 @@ impl AuthProvider for OpenIDConnectAuthProvider {
                 post_revocation_redirect_url,
                 ..
             } => {
-                if let Err(err) = self.try_revoke_token(&session) {
+                if let Err(err) = self.try_revoke_token(&session).await {
                     OpenIDConnectAuthProvider::internal_error(
                         format!("Error while revoking token for user '{}'", session.id),
                         Some(err.to_string()),
