@@ -69,19 +69,6 @@ impl Rfc8183Id {
     }
 }
 
-//------------ CertAuthStatus ----------------------------------------------
-#[derive(Clone, Copy, Debug, Deserialize, Serialize)]
-enum CertAuthStatus {
-    Active,
-    Deactivated,
-}
-
-impl Default for CertAuthStatus {
-    fn default() -> Self {
-        CertAuthStatus::Active
-    }
-}
-
 //------------ CertAuth ----------------------------------------------------
 
 /// This type defines a Certification Authority at a slightly higher level
@@ -102,17 +89,11 @@ pub struct CertAuth {
     children: HashMap<ChildHandle, ChildDetails>,
     routes: Routes,
 
-    #[serde(skip_serializing_if = "Rtas::is_empty", default = "Rtas::default")]
+    #[serde(skip_serializing_if = "Rtas::is_empty", default)]
     rtas: Rtas,
 
-    #[serde(
-        skip_serializing_if = "AspaDefinitions::is_empty",
-        default = "AspaDefinitions::default"
-    )]
+    #[serde(skip_serializing_if = "AspaDefinitions::is_empty", default)]
     aspas: AspaDefinitions,
-
-    #[serde(skip_serializing, default = "CertAuthStatus::default")]
-    status: CertAuthStatus,
 }
 
 impl Aggregate for CertAuth {
@@ -152,7 +133,6 @@ impl Aggregate for CertAuth {
             routes,
             rtas,
             aspas,
-            status: CertAuthStatus::Active,
         })
     }
 
