@@ -2,18 +2,21 @@ use std::ops::{Deref, DerefMut};
 
 use serde::{Deserialize, Serialize};
 
-use rpki::crypto::KeyIdentifier;
-use rpki::x509::Time;
+use rpki::repository::{crypto::KeyIdentifier, x509::Time};
 
-use crate::commons::api::{
-    ActiveInfo, CertifiedKeyInfo, EntitlementClass, Handle, IssuanceRequest, PendingInfo, PendingKeyInfo, RcvdCert,
-    RepoInfo, RequestResourceLimit, ResourceClassKeysInfo, ResourceClassName, ResourceSet, RevocationRequest,
-    RollNewInfo, RollOldInfo, RollPendingInfo,
+use crate::{
+    commons::{
+        api::{
+            ActiveInfo, CertifiedKeyInfo, EntitlementClass, Handle, IssuanceRequest, PendingInfo, PendingKeyInfo,
+            RcvdCert, RepoInfo, RequestResourceLimit, ResourceClassKeysInfo, ResourceClassName, ResourceSet,
+            RevocationRequest, RollNewInfo, RollOldInfo, RollPendingInfo,
+        },
+        crypto::KrillSigner,
+        error::Error,
+        KrillResult,
+    },
+    daemon::ca::CaEvtDet,
 };
-use crate::commons::crypto::KrillSigner;
-use crate::commons::error::Error;
-use crate::commons::KrillResult;
-use crate::daemon::ca::CaEvtDet;
 
 //------------ CertifiedKey --------------------------------------------------
 
@@ -92,24 +95,26 @@ impl CertifiedKey {
             return true;
         }
 
-        // If the validity time eligibility has changed, then we *may* want to ask for a new
-        // certificate, but only if:
-        // a) our current certificate expires *after* the eligible time, because we probably should
-        //    know..
-        // b) the new not after time is significantly better than our current time, because we
-        //    do not want to ask for new certificates every hour if the parent uses a simple
-        //    strategy like: not-after = now + 1 year..
+        // If the remaining validity time eligibility has changed by more than 10% then we will
+        // want to request a new certificate.
         //
-        // See issue #95
+        // We use this 10% margin to avoid ending up in endless request loops - in particular
+        // in cases where the parent uses a simple strategy like: not-after = now + 1 year for
+        // every list request we send. See issue #95.
+        //
+        // As it turns out there can also be timing issues with remaining time *reduction*. This
+        // is rather odd - as the parent is not really expected to reduce the time compared to
+        // what they issued to us before. But if it does happen (on every request like above)
+        // then we still want to avoid ending up in request loops. See issue #775
 
         let not_after = self.incoming_cert().cert().validity().not_after();
 
-        let now = Time::now().timestamp_millis();
-        let until_not_after_millis = not_after.timestamp_millis() - now;
-        let until_new_not_after_millis = new_not_after.timestamp_millis() - now;
+        let now = Time::now().timestamp();
+        let remaining_seconds_on_current = not_after.timestamp() - now;
+        let remaining_seconds_on_eligible = new_not_after.timestamp() - now;
 
-        if until_new_not_after_millis < 0 {
-            // New not after time is in the past!
+        if remaining_seconds_on_eligible <= 0 {
+            // eligible remaining time is in the past!
             //
             // This is rather odd. The parent should just exclude the resource class in the
             // eligible entitlements instead. So, we will essentially just ignore this until
@@ -121,24 +126,26 @@ impl CertifiedKey {
                 new_not_after.to_rfc3339()
             );
             false
-        } else if until_not_after_millis == until_new_not_after_millis {
+        } else if remaining_seconds_on_current == remaining_seconds_on_eligible {
             debug!(
-                "Will not request new certificate for CA '{}' under RC '{}'. Resources and not after time are unchanged\n",
+                "Will not request new certificate for CA '{}' under RC '{}'. Resources and not after time are unchanged.",
                 handle,
                 rcn,
             );
             false
-        } else if until_new_not_after_millis < until_not_after_millis {
+        } else if remaining_seconds_on_current > 0
+            && (remaining_seconds_on_eligible as f64 / remaining_seconds_on_current as f64) < 0.9_f64
+        {
             warn!(
-                "Parent of CA '{}' reduced not after time for certificate under RC '{}'",
+                "Parent of CA '{}' *reduced* not after time for certificate under RC '{}'. This is odd, but.. requesting new certificate.",
                 handle, rcn,
             );
             true
-        } else if until_not_after_millis <= 0
-            || (until_new_not_after_millis as f64 / until_not_after_millis as f64) > 1.1_f64
+        } else if remaining_seconds_on_current <= 0
+            || (remaining_seconds_on_eligible as f64 / remaining_seconds_on_current as f64) > 1.1_f64
         {
             info!(
-                "Will request new certificate for CA '{}' under RC '{}'. Not after time increased to: {}\n",
+                "Will request new certificate for CA '{}' under RC '{}'. Not after time increased to: {}",
                 handle,
                 rcn,
                 new_not_after.to_rfc3339()
@@ -146,9 +153,10 @@ impl CertifiedKey {
             true
         } else {
             debug!(
-                "Will not request new certificate for CA '{}' under RC '{}'. Not after time increased by less than 10%: {}\n",
+                "Will not request new certificate for CA '{}' under RC '{}'. Remaining not after time changed by less than 10%. From: {} To: {}",
                 handle,
                 rcn,
+                not_after.to_rfc3339(),
                 new_not_after.to_rfc3339()
             );
             false
