@@ -1,21 +1,24 @@
-use std::collections::HashMap;
-use std::fmt;
+use std::{collections::HashMap, fmt};
 
 use bytes::Bytes;
 
-use rpki::crypto::{DigestAlgorithm, KeyIdentifier};
-use rpki::rta;
-use rpki::sigobj::MessageDigest;
-use rpki::x509::Validity;
+use rpki::repository::{
+    crypto::{DigestAlgorithm, KeyIdentifier},
+    rta,
+    sigobj::MessageDigest,
+    x509::Validity,
+};
 
-use crate::commons::api::{Base64, ResourceClassName, ResourceSet, Revocation, RtaList, RtaName};
-use crate::commons::error::Error;
-use crate::commons::util::ext_serde;
-use crate::commons::KrillResult;
+use crate::commons::{
+    api::{Base64, ResourceClassName, ResourceSet, Revocation, RtaList, RtaName},
+    error::Error,
+    util::ext_serde,
+    KrillResult,
+};
 
 //------------ Rtas ---------------------------------------------------------
 
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub struct Rtas {
     map: HashMap<RtaName, RtaState>,
 }
@@ -58,12 +61,6 @@ impl Rtas {
     }
 }
 
-impl Default for Rtas {
-    fn default() -> Self {
-        Rtas { map: HashMap::new() }
-    }
-}
-
 //------------ RtaState -----------------------------------------------------
 
 #[derive(Clone, Debug, Deserialize, Serialize)]
