@@ -8,7 +8,6 @@ use std::{
 };
 
 use chrono::Duration;
-use clap::{App, Arg};
 use log::{error, LevelFilter};
 use serde::{de, Deserialize, Deserializer};
 
@@ -30,6 +29,8 @@ use crate::{
 #[cfg(feature = "multi-user")]
 use crate::daemon::auth::providers::{config_file::config::ConfigAuthUsers, openid_connect::ConfigAuthOpenIDConnect};
 
+use super::mq::{in_seconds, Priority};
+
 //------------ ConfigDefaults ------------------------------------------------
 
 pub struct ConfigDefaults;
@@ -105,8 +106,12 @@ impl ConfigDefaults {
         vec![]
     }
 
-    fn ca_refresh_seconds() -> u32 {
-        600
+    fn ca_refresh_seconds() -> i64 {
+        24 * 3600 // 24 hours
+    }
+
+    fn ca_refresh_jitter_seconds() -> i64 {
+        12 * 3600 // 12 hours
     }
 
     fn ca_refresh_parents_batch_size() -> usize {
@@ -133,6 +138,10 @@ impl ConfigDefaults {
         None
     }
 
+    fn post_protocol_msg_timeout_seconds() -> u64 {
+        240 // 4 minutes by default should be plenty in most cases
+    }
+
     fn bgp_risdumps_enabled() -> bool {
         true
     }
@@ -264,7 +273,10 @@ pub struct Config {
     pub auth_openidconnect: Option<ConfigAuthOpenIDConnect>,
 
     #[serde(default = "ConfigDefaults::ca_refresh_seconds", alias = "ca_refresh")]
-    pub ca_refresh_seconds: u32,
+    ca_refresh_seconds: i64,
+
+    #[serde(default = "ConfigDefaults::ca_refresh_jitter_seconds")]
+    ca_refresh_jitter_seconds: i64,
 
     #[serde(default = "ConfigDefaults::ca_refresh_parents_batch_size")]
     pub ca_refresh_parents_batch_size: usize,
@@ -285,6 +297,9 @@ pub struct Config {
     #[serde(default = "ConfigDefaults::post_limit_rfc6492")]
     pub post_limit_rfc6492: u64,
 
+    #[serde(default = "ConfigDefaults::post_protocol_msg_timeout_seconds")]
+    pub post_protocol_msg_timeout_seconds: u64,
+
     #[serde(default = "ConfigDefaults::rfc6492_log_dir")]
     pub rfc6492_log_dir: Option<PathBuf>,
 
@@ -530,9 +545,45 @@ impl Config {
         }
     }
 
+    pub fn requeue_remote_failed(&self) -> Priority {
+        if test_mode_enabled() {
+            in_seconds(5)
+        } else {
+            in_seconds(SCHEDULER_REQUEUE_DELAY_SECONDS)
+        }
+    }
+
+    /// Get the priority for the next CA refresh based on the configured
+    /// ca_refresh_seconds (1 day), and jitter (12 hours)
+    pub fn ca_refresh_next(&self) -> Priority {
+        Self::ca_refresh_next_from(self.ca_refresh_seconds, self.ca_refresh_jitter_seconds)
+    }
+
+    pub fn ca_refresh_start_up(&self, use_jitter: bool) -> Priority {
+        let jitter_seconds = if use_jitter { self.ca_refresh_jitter_seconds } else { 0 };
+
+        Self::ca_refresh_next_from(0, jitter_seconds)
+    }
+
+    fn ca_refresh_next_from(regular_seconds: i64, jitter_seconds: i64) -> Priority {
+        let random_seconds = if jitter_seconds == 0 {
+            0
+        } else {
+            use rand::Rng;
+            let mut rng = rand::thread_rng();
+            rng.gen_range(0..jitter_seconds)
+        };
+
+        in_seconds(regular_seconds + random_seconds)
+    }
+
     pub fn testbed(&self) -> Option<&TestBed> {
         self.testbed.as_ref()
     }
+
+    pub fn upgrade_data_dir(&self) -> PathBuf {
+        self.data_dir.join("upgrade-data")
+    }
 }
 
 /// # Create
@@ -564,6 +615,7 @@ impl Config {
         #[cfg(feature = "multi-user")]
         let auth_openidconnect = None;
         let ca_refresh_seconds = if enable_ca_refresh { 1 } else { 86400 };
+        let ca_refresh_jitter_seconds = if enable_ca_refresh { 0 } else { 86400 }; // no jitter in testing
         let ca_refresh_parents_batch_size = 10;
         let post_limit_api = ConfigDefaults::post_limit_api();
         let post_limit_rfc8181 = ConfigDefaults::post_limit_rfc8181();
@@ -578,6 +630,7 @@ impl Config {
             dir.push("rfc6492");
             Some(dir)
         };
+        let post_protocol_msg_timeout_seconds = ConfigDefaults::post_protocol_msg_timeout_seconds();
 
         let bgp_risdumps_enabled = false;
         let bgp_risdumps_v4_uri = ConfigDefaults::bgp_risdumps_v4_uri();
@@ -661,6 +714,7 @@ impl Config {
             #[cfg(feature = "multi-user")]
             auth_openidconnect,
             ca_refresh_seconds,
+            ca_refresh_jitter_seconds,
             ca_refresh_parents_batch_size,
             suspend_child_after_inactive_seconds,
             suspend_child_after_inactive_hours: None,
@@ -669,6 +723,7 @@ impl Config {
             rfc8181_log_dir,
             post_limit_rfc6492,
             rfc6492_log_dir,
+            post_protocol_msg_timeout_seconds,
             bgp_risdumps_enabled,
             bgp_risdumps_v4_uri,
             bgp_risdumps_v6_uri,
@@ -691,68 +746,58 @@ impl Config {
         config
     }
 
-    pub fn get_config_filename() -> String {
-        let matches = App::new(KRILL_SERVER_APP)
-            .version(KRILL_VERSION)
-            .arg(
-                Arg::with_name("config")
-                    .short("c")
-                    .long("config")
-                    .value_name("FILE")
-                    .help("Override the path to the config file (default: './defaults/krill.conf')")
-                    .required(false),
-            )
-            .get_matches();
+    /// Creates the config (at startup).
+    pub fn create(config_file: &str, upgrade_only: bool) -> Result<Self, ConfigError> {
+        let mut config = Self::read_config(config_file)?;
 
-        let config_file = matches.value_of("config").unwrap_or(KRILL_DEFAULT_CONFIG_FILE);
+        if upgrade_only {
+            config.log_type = LogType::Stderr;
+        }
 
-        config_file.to_string()
+        config.init_logging()?;
+
+        if upgrade_only {
+            info!("Prepare upgrade using configuration file: {}", config_file);
+            info!("Processing data from: {}", config.data_dir.to_string_lossy());
+            info!(
+                "Saving prepared data to: {}",
+                config.upgrade_data_dir().to_string_lossy()
+            );
+        } else {
+            info!("{} uses configuration file: {}", KRILL_SERVER_APP, config_file);
+        }
+
+        config.trim_refresh()?;
+        config.verify()?;
+
+        Ok(config)
     }
 
-    /// Creates the config (at startup). Panics in case of issues.
-    pub fn create() -> Result<Self, ConfigError> {
-        let config_file = Self::get_config_filename();
-
-        let mut config = match Self::read_config(&config_file) {
-            Err(e) => {
-                if config_file == KRILL_DEFAULT_CONFIG_FILE {
-                    Err(ConfigError::other(
-                        "Cannot find config file. Please use --config to specify its location.",
-                    ))
-                } else {
-                    Err(ConfigError::Other(format!(
-                        "Error parsing config file: {}, error: {}",
-                        config_file, e
-                    )))
-                }
-            }
-            Ok(config) => {
-                config.init_logging()?;
-                info!("{} uses configuration file: {}", KRILL_SERVER_APP, config_file);
-                Ok(config)
-            }
-        }?;
-
-        if config.ca_refresh_seconds < CA_REFRESH_SECONDS_MIN {
+    fn trim_refresh(&mut self) -> Result<(), ConfigError> {
+        if self.ca_refresh_seconds < CA_REFRESH_SECONDS_MIN {
             warn!(
                 "The value for 'ca_refresh_seconds' was below the minimum value, changing it to {} seconds",
                 CA_REFRESH_SECONDS_MIN
             );
-            config.ca_refresh_seconds = CA_REFRESH_SECONDS_MIN;
+            self.ca_refresh_seconds = CA_REFRESH_SECONDS_MIN;
         }
 
-        if config.ca_refresh_seconds > CA_REFRESH_SECONDS_MAX {
+        if self.ca_refresh_seconds > CA_REFRESH_SECONDS_MAX {
             warn!(
                 "The value for 'ca_refresh_seconds' was above the maximum value, changing it to {} seconds",
                 CA_REFRESH_SECONDS_MAX
             );
-            config.ca_refresh_seconds = CA_REFRESH_SECONDS_MAX;
+            self.ca_refresh_seconds = CA_REFRESH_SECONDS_MAX;
         }
 
-        config
-            .verify()
-            .map_err(|e| ConfigError::Other(format!("Error parsing config file: {}, error: {}", config_file, e)))?;
-        Ok(config)
+        let half_refresh = self.ca_refresh_seconds / 2;
+
+        if self.ca_refresh_jitter_seconds > half_refresh {
+            warn!("The value for 'ca_refresh_jitter_seconds' exceeded 50% of 'ca_refresh_seconds'. Changing it to {} seconds", half_refresh);
+            self.ca_refresh_jitter_seconds = half_refresh;
+        }
+
+        Ok(())
     }
 
     pub fn verify(&self) -> Result<(), ConfigError> {
@@ -849,13 +894,20 @@ impl Config {
 
     pub fn read_config(file: &str) -> Result<Self, ConfigError> {
         let mut v = Vec::new();
-        let mut f =
-            File::open(file).map_err(|e| KrillIoError::new(format!("Could not read open file '{}'", file), e))?;
+        let mut f = File::open(file).map_err(|e| {
+            KrillIoError::new(
+                format!(
+                    "Could not read config file '{}'. Note: you may want to override the default location using --config <path>",
+                    file
+                ),
+                e,
+            )
+        })?;
         f.read_to_end(&mut v)
             .map_err(|e| KrillIoError::new(format!("Could not read config file '{}'", file), e))?;
 
-        let c: Config = toml::from_slice(v.as_slice())?;
-        Ok(c)
+        toml::from_slice(v.as_slice())
+            .map_err(|e| ConfigError::Other(format!("Error parsing config file: {}, error: {}", file, e)))
     }
 
     pub fn init_logging(&self) -> Result<(), ConfigError> {
