@@ -6,7 +6,6 @@ extern crate bcder;
 extern crate bytes;
 extern crate chrono;
 extern crate clap;
-extern crate clokwerk;
 extern crate futures;
 extern crate futures_util;
 extern crate hex;
@@ -16,6 +15,7 @@ extern crate libflate;
 #[macro_use]
 extern crate log;
 extern crate openssl;
+extern crate priority_queue;
 extern crate rand;
 extern crate reqwest;
 extern crate rpki;
