@@ -32,7 +32,7 @@ use crate::{
     },
     constants::{
         PUBSERVER_CONTENT_DIR, PUBSERVER_DFLT, PUBSERVER_DIR, REPOSITORY_DIR, REPOSITORY_RRDP_ARCHIVE_DIR,
-        REPOSITORY_RRDP_DIR, REPOSITORY_RSYNC_DIR,
+        REPOSITORY_RRDP_DIR, REPOSITORY_RSYNC_DIR, RRDP_FIRST_SERIAL,
     },
     daemon::config::{Config, RepositoryRetentionConfig},
     pubd::{
@@ -74,6 +74,7 @@ impl RepositoryContentProxy {
             info!("Warming the repository content cache, this can take a minute for large repositories.");
             let content = key_store_read.get(&self.key)?.unwrap();
             self.cache.write().unwrap().replace(content);
+            info!("Repository content cache has been warmed.");
         }
 
         Ok(())
@@ -551,7 +552,7 @@ impl RrdpServer {
 
         let snapshot = Snapshot::create(session);
 
-        let serial = 0;
+        let serial = RRDP_FIRST_SERIAL;
         let snapshot_uri = snapshot.uri(&rrdp_base_uri);
         let snapshot_path = snapshot.path(&rrdp_base_dir);
         let snapshot_hash = HexEncodedHash::from_content(snapshot.xml().as_slice());
