@@ -1,4 +1,4 @@
-use rpki::{crypto::KeyIdentifier, x509::Time};
+use rpki::repository::{crypto::KeyIdentifier, x509::Time};
 
 use std::{
     collections::{BTreeMap, HashMap},
@@ -17,11 +17,10 @@ use crate::{
     },
     daemon::ca::StoredCaCommand,
     pubd::RepositoryManager,
-    upgrades::{UpgradeError, UpgradeResult},
+    upgrades::v0_9_0::old_events::DerivedEmbeddedCaMigrationInfo,
+    upgrades::{PrepareUpgradeError, UpgradeResult},
 };
 
-use super::old_events::DerivedEmbeddedCaMigrationInfo;
-
 #[derive(Clone, Debug, Deserialize, Eq, PartialEq, Serialize)]
 pub struct OldStoredCaCommand {
     pub actor: String,
@@ -63,7 +62,7 @@ impl OldStoredCaCommand {
                     None => derived_embedded_ca_info_map
                         .get(&child)
                         .ok_or_else(|| {
-                            UpgradeError::Custom(format!(
+                            PrepareUpgradeError::Custom(format!(
                                 "Cannot upgrade CA history for {}, child {} is no longer present",
                                 handle, child
                             ))
